# generated by datamodel-codegen:
#   filename:  Entity.json

from __future__ import annotations

from pydantic.v1 import Field

from oold.model.v1 import LinkedBaseModel


class SimpleSubSchema(LinkedBaseModel):
    class Config:
        schema_extra = {"title": "SimpleSubSchema"}

    some_property: str | None = None


class NestedSubSchema(LinkedBaseModel):
    """
    An example subschema
    """

    class Config:
        schema_extra = {
            "title": "NestedSubSchema",
            "description": "An example subschema",
        }

    subschema: SimpleSubSchema | None = Field(None, title="SimpleSubSchema")


class Entity(LinkedBaseModel):
    class Config:
        schema_extra = {"title": "Entity"}

    nested: NestedSubSchema | None = Field(
        None, title="EntityNestedSubSchema", x_custom_annotation="custom value"
    )
    """
    An example using a subschema
    """
