# generated by datamodel-codegen:
#   filename:  Entity.json

from __future__ import annotations

from pydantic import ConfigDict, Field

from oold.model import LinkedBaseModel


class SimpleSubSchema(LinkedBaseModel):
    model_config = ConfigDict(
        json_schema_extra={"title": "SimpleSubSchema"},
    )
    some_property: str | None = None


class NestedSubSchema(LinkedBaseModel):
    """
    An example subschema
    """

    model_config = ConfigDict(
        json_schema_extra={
            "title": "NestedSubSchema",
            "description": "An example subschema",
        },
    )
    subschema: SimpleSubSchema | None = Field(None, title="SimpleSubSchema")


class Entity(LinkedBaseModel):
    model_config = ConfigDict(
        json_schema_extra={"title": "Entity"},
    )
    nested: NestedSubSchema | None = Field(
        None,
        json_schema_extra={"x-custom-annotation": "custom value"},
        title="EntityNestedSubSchema",
    )
    """
    An example using a subschema
    """
